#-----------------------------------------------------------------------------
# Restore feet
# Mar 16, 2006
# Goes with the MemFeet.py script.  MemFeet gathers the actual center of
# the feet on the current figure and stores in file "feet.dat".
# ResFeet picks up that file (if present) and moves the current figure
# so its feet are centered in the same place.

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

import poser
import pickle
from Tkinter import *
import os

#-----------------------------------------------------------------------------

scene=poser.Scene()
CurFig=scene.CurrentFigure()
if not CurFig: raise "No figure selected"

BasePathUncut = poser.AppLocation()
BasePath = BasePathUncut[:(-len("poser.exe"))]
DatName=os.path.join(BasePath,'runtime','feet.dat')


#-----------------------------------------------------------------------------
# Next section is TK
#-----------------------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        self.master = master
        master.title("Restore feet")

        self.StatusEntry = Entry(self.master,width=50)
        self.StatusEntry.grid(row=0,column=0,columnspan=2)

        # Go/nogo buttons.
        self.buttonLeftOnLeft = Button(self.master, text="LeftOnLeft", command=self.HandleLeftOnLeft)
        self.buttonLeftOnLeft.grid(row=2, column=0)

        self.buttonLeftOnRight = Button(self.master, text="Left On Right", command=self.HandleLeftOnRight)
        self.buttonLeftOnRight.grid(row=2, column=1)

        self.buttonRightOnRight = Button(self.master, text="Right On Right", command=self.HandleRightOnRight)
        self.buttonRightOnRight.grid(row=3, column=0)

        self.buttonRightOnLeft = Button(self.master, text="Right On Left", command=self.HandleRightOnLeft)
        self.buttonRightOnLeft.grid(row=3, column=1)

        self.buttonCenterOnCenter = Button(self.master, text="Center On Center", command=self.HandleCenterOnCenter)
        self.buttonCenterOnCenter.grid(row=4, column=0)

        self.buttonCancel = Button(self.master, text="Cancel", command=self.HandleCancel)
        self.buttonCancel.grid(row=4, column=1)

        self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)

    # - - - - - - - - - - - - - - - - - -

    def InitStuff(self):
        # Check things
        try:
            Lfoot=CurFig.ActorByInternalName("lFoot")
            Rfoot=CurFig.ActorByInternalName("rFoot")
        except:
            raise "Figure doesn't have feet"
        try:
            Lgeom=Lfoot.Geometry()
            Rgeom=Rfoot.Geometry()
        except:
            self.ShowStatus("No mesh?")
            return

        # Form average center
        scene.DrawAll()
        LMaxX=-10000.0
        LMaxZ=-10000.0
        LMinX= 10000.0
        LMinZ= 10000.0
        RMaxX=-10000.0
        RMaxZ=-10000.0
        RMinX= 10000.0
        RMinZ= 10000.0
        for v in range(Lgeom.NumVertices()):
            LWV=Lgeom.WorldVertex(v)
            if   LWV.X()>LMaxX: LMaxX=LWV.X()
            elif LWV.X()<LMinX: LMinX=LWV.X()
            if   LWV.Z()>LMaxZ: LMaxZ=LWV.Z()
            elif LWV.Z()<LMinZ: LMinZ=LWV.Z()
            RWV=Rgeom.WorldVertex(v)
            if   RWV.X()>RMaxX: RMaxX=RWV.X()
            elif RWV.X()<RMinX: RMinX=RWV.X()
            if   RWV.Z()>RMaxZ: RMaxZ=RWV.Z()
            elif RWV.Z()<RMinZ: RMinZ=RWV.Z()

        self.LX=(LMaxX+LMinX)/2.0
        self.LZ=(LMaxZ+LMinZ)/2.0
        self.RX=(RMaxX+RMinX)/2.0
        self.RZ=(RMaxZ+RMinZ)/2.0

        # Get the starting location of body.
        Bod=CurFig.Actor("BODY")
        self.BodXparm=Bod.ParameterByCode(poser.kParmCodeXTRAN)
        self.BodZparm=Bod.ParameterByCode(poser.kParmCodeZTRAN)

        # Now we have the actual center of the feet.
        # Get the desired position:
        try:
            fp=open(DatName,"rt")
        except:
            raise "No stored file."
        Desired=pickle.load(fp)
        fp.close()
        self.DesPos=Desired # Do this because pickle doesn't like self.vars

    # - - - - - - - - - - - - - - - - - -

    def RonR(self):
        # Move so right matches desired right.
        (DesLX,DesLZ,DesRX,DesRZ)=self.DesPos
        # Get the starting location of body.
        dX=self.RX-self.BodXparm.Value()
        dZ=self.RZ-self.BodZparm.Value()
        self.BodXparm.SetValue(DesRX-dX)
        self.BodZparm.SetValue(DesRZ-dZ)

    # - - - - - - - - - - - - - - - - - -

    def RonL(self):
        # Move so right matches desired left.
        (DesLX,DesLZ,DesRX,DesRZ)=self.DesPos
        # Get the starting location of body.
        dX=self.RX-self.BodXparm.Value()
        dZ=self.RZ-self.BodZparm.Value()
        self.BodXparm.SetValue(DesLX-dX)
        self.BodZparm.SetValue(DesLZ-dZ)

    # - - - - - - - - - - - - - - - - - -

    def LonL(self):
        # Move so left matches desired left.
        (DesLX,DesLZ,DesRX,DesRZ)=self.DesPos
        # Get the starting location of body.
        dX=self.LX-self.BodXparm.Value()
        dZ=self.LZ-self.BodZparm.Value()
        self.BodXparm.SetValue(DesLX-dX)
        self.BodZparm.SetValue(DesLZ-dZ)

    # - - - - - - - - - - - - - - - - - -

    def LonR(self):
        # Move so left matches desired right.
        (DesLX,DesLZ,DesRX,DesRZ)=self.DesPos
        # Get the starting location of body.
        dX=self.LX-self.BodXparm.Value()
        dZ=self.LZ-self.BodZparm.Value()
        self.BodXparm.SetValue(DesRX-dX)
        self.BodZparm.SetValue(DesRZ-dZ)

    # - - - - - - - - - - - - - - - - - -

    def ConC(self):
        # Move so center matches desired center.
        (DesLX,DesLZ,DesRX,DesRZ)=self.DesPos
        # Get the starting location of body.
        dX=self.LX-self.BodXparm.Value()
        dZ=self.LZ-self.BodZparm.Value()

        DesCX=(DesLX+DesRX)/2.0
        DesCZ=(DesLZ+DesRZ)/2.0

        self.CX=(self.LX+self.RX)/2.0
        self.CZ=(self.LZ+self.RZ)/2.0
        dX=self.CX-self.BodXparm.Value()
        dZ=self.CZ-self.BodZparm.Value()

        self.BodXparm.SetValue(DesCX-dX)
        self.BodZparm.SetValue(DesCZ-dZ)

    # - - - - - - - - - - - - - - - - - -

    def ShowStatus(self,S): # Just saving repetition
        self.StatusEntry.delete(0,END)
        self.StatusEntry.insert(0,S)
        self.StatusEntry.update_idletasks()

    # - - - - - - - - - - - - - - - - - -

    def Update(self):
        if scene: scene.ProcessSomeEvents()
        root.lift()
        root.after(100, self.Update)

    # - - - - - - - - - - - - - - - - - -

    def HandleLeftOnLeft(self):
        self.InitStuff()
        self.LonL()
        self.master.destroy()

    # - - - - - - - - - - - - - - - - - -

    def HandleLeftOnRight(self):
        self.InitStuff()
        self.LonR()
        self.master.destroy()

    # - - - - - - - - - - - - - - - - - -

    def HandleRightOnRight(self):
        self.InitStuff()
        self.RonR()
        self.master.destroy()

    # - - - - - - - - - - - - - - - - - -

    def HandleRightOnLeft(self):
        self.InitStuff()
        self.RonL()
        self.master.destroy()

    # - - - - - - - - - - - - - - - - - -

    def HandleCenterOnCenter(self):
        self.InitStuff()
        self.ConC()
        self.master.destroy()

    # - - - - - - - - - - - - - - - - - -

    def HandleCancel(self):
        self.master.destroy()

    # - - - - - - - - - - - - - - - - - -

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#------------------------------------------------------------
